base = (typeof moduleData !== 'undefined' && moduleData.exports) ? root : window
base.documentorData = {
  "id": "dd_1",
  "files": [],
  "classes": [
    {
      "id": "dd_2",
      "name": "ClickToEditForm",
      "comment": [
        "",
        "This component is an extension of **Form** that initially presents an",
        "\"Edit\" button that the user can click to switch between display and input.",
        "",
        "**Datum** children are initialized readonly, when the user clicks edit, the",
        "'inputMode' context variable is set to 'edit' and all children are rerendered.",
        "",
        "See react-datum **Form** component for more props.  All **Form** properties are",
        "supported by **ClickToEditForm**",
        ""
      ],
      "lineNumber": 5,
      "file": "src/clickToEditForm.cjsx",
      "signature": "module.exports = class ClickToEditForm extends Form",
      "superClass": " Form",
      "methods": []
    },
    {
      "id": "dd_11",
      "name": "Collection",
      "comment": [
        "",
        "This react-datum component provides datums with a Backbone collection and rerenders descendants on collection changes.  ",
        "- provides a collection context to all children",
        "- rerenders children on collection changes",
        "- adds SelectableCollection mixin to collection if it doesn't already have it",
        "- will optionally fetch the collection",
        "- can accept either a Collection class (which will be instantiated) or a",
        "collection instance variable to another collection or Collection component",
        "",
        "",
        "Any descendant datum or react-datum based components like (ReactDatum.TileGrid)[TODO: add a link do doc page], or the (ReactDatum.SelectedModel)[TODO: add link to doc page] that accept a `collection` context or prop, can be wrapped in a ReactDatum.Collection component and will respond to changes in the collection.  ",
        "",
        "Simple example:",
        "```jsx",
        "  var Rd = ReactDatum;",
        "  ",
        "  <Rd.Collection collection={myBackboneCollection}>",
        "    <Rd.SelectedModel placeholder=\"Wait for it...\">",
        "      <Rd.Text label=\"You selected: \" attr=\"name\"/>",
        "    </Rd.SelectedModel>",
        "  </Rd>",
        "  ",
        "  var index = 0;",
        "  setInterval(function(){",
        "    myBackboneCollection.selectModelByIndex(index);",
        "    index += 1;",
        "  }, 5000)",
        "",
        "```",
        "The (example above)[TODO: add link to github.io example] when run, will initially display a placeholder, \"Wait for it...\" and then every five seconds select a new model. See (SelectedModel)[TODO: add link to doc page] for more information about selections.",
        "",
        "#### Adds selection tracking to collection",
        "",
        "In order for the components such as ReactDatum.Tilegrid and ReactDatum.SelectedModel to have a common understanding of what is selected,  a small mixin called SelectableCollection is added to the collection, unless the collection has an instance variable hasSelectableCollectionMixin=`true`. ",
        "",
        "Once used with the ReactDatum.Collection component, your Backbone collection instance will be given the following methods:",
        "",
        "  * `getSelectedModels()`  returns array of currently selected models",
        "  * `selectModel(model, selected=true, options={})`  selects the model. default options: {silent: false}",
        "  * `selectModelById(id, selected=true, options={})`  see selectModel()",
        "  * `selectModelByIndex(index, selected=true, options={})`  see selectModel()",
        "  * `selectAll(options={})` default options: {silent: false}",
        "  * `selectNone(options{})` default options: {silent: false}",
        "",
        "A console warning will be issued if any of the methods above already exist on a collection without this.hasSelectableCollectionMixin==true.   ",
        "",
        "If you use another mixin or service for this, it should be relatively easy to build a bridge. Just add the methods above to your collection and call the existing plugin, mixin or external methods.",
        "",
        "",
        ""
      ],
      "lineNumber": 10,
      "file": "src/collection.cjsx",
      "signature": "module.exports = class Collection extends ContextualData",
      "superClass": " ContextualData",
      "methods": [],
      "propTypes": [
        "  @propTypes: _.extend {}, ContextualData.propTypes,",
        "    # can only accept collection via prop.  Collection component should",
        "    # not interfere with any parent Collection commponent or other that",
        "    # would provide a collection context, but it will become the provider",
        "    # of collection context for all of it's children",
        "    collection: React.PropTypes.oneOfType([",
        "      React.PropTypes.instanceOf(Backbone.Collection)",
        "      React.PropTypes.func",
        "    ]).isRequired"
      ]
    },
    {
      "id": "dd_14",
      "name": "CollectionStats",
      "comment": [
        "",
        "  - provides information about the items in your collection.",
        "",
        "  Requires either a 'collection' context or prop.  And displays counts of",
        "  items found, selected and viewing.",
        "",
        "",
        ""
      ],
      "lineNumber": 5,
      "file": "src/collectionStats.cjsx",
      "signature": "module.exports = class CollectionStats extends React.Component",
      "superClass": " React.Component",
      "methods": [],
      "propTypes": [
        "  @propTypes:",
        "    # a Backbone.Collection",
        "    collection: React.PropTypes.instanceOf(Backbone.Collection)"
      ],
      "defaultProps": [
        "  @defaultProps:",
        "    itemDisplayName: \"item\""
      ],
      "contextTypes": [
        "  @contextTypes:",
        "    collection: React.PropTypes.instanceOf(Backbone.Collection)"
      ]
    },
    {
      "id": "dd_19",
      "name": "CollectionPicker",
      "comment": [
        "",
        "This react-datum component provides a selector (picker) based on react-select.  That looks up an ID, such",
        "as maybe a userID on a model in another collection such as a Users collection.",
        "",
        "simple example:",
        "```jsx",
        "  <ReactDatum.CollectionPicker ",
        "    attr=\"userId\"   ",
        "    model={myModel}",
        "    collection={usersCollection}",
        "    displayAttr=\"fullName\"/>",
        "```",
        "",
        "In display mode, the CollectionPicker above will display the fullName from a model with ID matching",
        "myModel.get('userId') from usersCollection instance var.   ",
        "",
        "In inputMode='edit', the suggestions would also render the displayAttr because we didn't specify a ",
        "suggestionAttr prop",
        "",
        "#### Asynchronous suggestions",
        "",
        "Use the asyncLoadCallback prop to load suggestions asynchronously from a server:",
        "```jsx",
        "  loadData = function(userInput, doneCallback) {",
        "    usersCollection.fetch({",
        "      data: {search: userInput}",
        "      success: function(model. response) {",
        "        # doneCallback accepts error and collection, array of models, ",
        "        # or array of {label: \"displan name\", value: \"value\"} ",
        "        doneCallback(null, usersCollection)",
        "      }",
        "    )",
        "  }",
        "  <ReactDatum.CollectionPicker ",
        "    attr=\"userId\" ",
        "    model=\"myOtherModel\" ",
        "    collection=\"usersCollection\" ",
        "    asyncLoadCallback={loadData} ",
        "  />",
        "```",
        "#### Custom display",
        "",
        "By default the CollectionPicker will display the value of props.model.get(props.displayAttr),",
        "or model.toString() (if it exists and no displayAttr specified).  ",
        "",
        "You can provide custom rendering via the the `optionComponent`, `valueComponent` and ",
        "`displayComponent` props.  ",
        "",
        "The optionComponent and valueComponent are passed throught to ",
        "[react-select](http://jedwatson.github.io/react-select/).  For optionComponent, you are ",
        "required to handle and pass events back through the props given. Providing a custom option ",
        "render, is a little longish, but value display is easy. The GravatarOption and GravatarValue",
        "components below were taken directly from the [CustomComponents example](https://github.com/JedWatson/react-select/blob/master/examples/src/components/CustomComponents.js)",
        "",
        "To provide a custom option (suggestion) display:",
        "",
        "```jsx",
        "  const GravatarOption = React.createClass({",
        "  \tpropTypes: {",
        "  \t\tclassName: React.PropTypes.string,",
        "  \t\tisDisabled: React.PropTypes.bool,",
        "  \t\tisFocused: React.PropTypes.bool,",
        "  \t\tisSelected: React.PropTypes.bool,",
        "  \t\tonSelect: React.PropTypes.func,",
        "  \t\tonFocus: React.PropTypes.func,",
        "  \t\tonUnfocus: React.PropTypes.func,",
        "  \t\toption: React.PropTypes.object.isRequired,",
        "  \t},",
        "  \thandleMouseDown (event) {",
        "  \t\tevent.preventDefault();",
        "  \t\tevent.stopPropagation();",
        "  \t\tthis.props.onSelect(this.props.option, event);",
        "  \t},",
        "  \thandleMouseEnter (event) {",
        "  \t\tthis.props.onFocus(this.props.option, event);",
        "  \t},",
        "  \thandleMouseMove (event) {",
        "  \t\tif (this.props.focused) return;",
        "  \t\tthis.props.onFocus(this.props.option, event);",
        "  \t},",
        "  \thandleMouseLeave (event) {",
        "  \t\tthis.props.onUnfocus(this.props.option, event);",
        "  \t},",
        "  \trender () {",
        "  \t\tlet gravatarStyle = {",
        "  \t\t\tborderRadius: 3,",
        "  \t\t\tdisplay: 'inline-block',",
        "  \t\t\tmarginRight: 10,",
        "  \t\t\tposition: 'relative',",
        "  \t\t\ttop: -2,",
        "  \t\t\tverticalAlign: 'middle',",
        "  \t\t};",
        "  \t\treturn (",
        "  \t\t\t<div className={this.props.className}",
        "  \t\t\t\tonMouseDown={this.handleMouseDown}",
        "  \t\t\t\tonMouseEnter={this.handleMouseEnter}",
        "  \t\t\t\tonMouseMove={this.handleMouseMove}",
        "  \t\t\t\tonMouseLeave={this.handleMouseLeave}",
        "  \t\t\t\ttitle={this.props.option.title}>",
        "  \t\t\t\t<Gravatar email={this.props.option.email} size={GRAVATAR_SIZE} style={gravatarStyle} />",
        "  \t\t\t\t{this.props.children}",
        "  \t\t\t</div>",
        "  \t\t);",
        "  \t}",
        "  });",
        "",
        "  <CollectionPicker",
        "    model={myModel}",
        "    collection={usersCollection}",
        "    optionComponent={GravatarOption}",
        "    placeholder=\"Select user\"",
        "  />",
        "```",
        "",
        "You can also provide a custom value display for the current value displayed when the input doesn't",
        "have focus:",
        "```jsx",
        "",
        "const GravatarValue = React.createClass({",
        "\tpropTypes: {",
        "\t\tplaceholder: React.PropTypes.string,",
        "\t\tvalue: React.PropTypes.object",
        "\t},",
        "\trender () {",
        "\t\tvar gravatarStyle = {",
        "\t\t\tborderRadius: 3,",
        "\t\t\tdisplay: 'inline-block',",
        "\t\t\tmarginRight: 10,",
        "\t\t\tposition: 'relative',",
        "\t\t\ttop: -2,",
        "\t\t\tverticalAlign: 'middle',",
        "\t\t};",
        "\t\treturn (",
        "\t\t\t<div className=\"Select-value\" title={this.props.value.title}>",
        "\t\t\t\t<span className=\"Select-value-label\">",
        "\t\t\t\t\t<Gravatar email={this.props.value.email} size={GRAVATAR_SIZE} style={gravatarStyle} />",
        "\t\t\t\t\t{this.props.children}",
        "\t\t\t\t</span>",
        "\t\t\t</div>",
        "\t\t);",
        "\t}",
        "});",
        "",
        "",
        "<CollectionPicker",
        "  model={myModel}",
        "  collection={usersCollection}",
        "  valueComponent={GravatarValue}",
        "  placeholder=\"Select user\"",
        "/>",
        "```",
        "",
        "Custom display when CollectionPicker is in inputMode=\"readonly\" is even easier:",
        "",
        "TODO:  finish me!",
        "",
        "",
        "",
        "",
        "",
        " !See ./collectionPicker.md "
      ],
      "lineNumber": 11,
      "file": "src/datums/collectionPicker/collectionPicker.cjsx",
      "signature": "module.exports = class CollectionPicker extends Datum",
      "superClass": " Datum",
      "methods": [
        {
          "id": "dd_30",
          "name": "filterSuggestionModels",
          "comment": [
            "   weak string compare userInput to suggestion model's display value "
          ],
          "lineNumber": 239,
          "file": "src/datums/collectionPicker/collectionPicker.cjsx",
          "signature": "  filterSuggestionModels: (collection, userInput, callback) =>"
        }
      ],
      "propTypes": [
        "  @propTypes: _.extend {}, Datum.propTypes,",
        "    ",
        "    #  TBD: Can also be the string name of a shared collection (see ../sharedCollection.cjsx)",
        "    #  TBD: Can also accept an array of [{lable: \"option 1\", id: 1}, ...]",
        "    #  ",
        "    #  Can also accept collection instance as context var via ReactDatum.Collection component. ",
        "    #    prop has precendence",
        "    collection: React.PropTypes.oneOfType([",
        "      React.PropTypes.instanceOf(Backbone.Collection)",
        "      React.PropTypes.string",
        "      React.PropTypes.array",
        "    ])",
        "    ",
        "    #  Attribute value from model in lookup collection to render in inputMode='readonly'.",
        "    #  if not specified, model.toString() will be displayed",
        "    displayAttr: React.PropTypes.string",
        "    ",
        "    #  attribute value from model in lookup collection to render in suggestions when ",
        "    #  in inputMode='edit'.  If not specified, @props.displayAttr is used and if that",
        "    #  is not specified, model.toString() is used",
        "    optionDisplayAttr: React.PropTypes.string"
      ],
      "defaultProps": [
        "  @defaultProps: _.extend {}, Datum.defaultProps,",
        "    optionSaveAttr: 'id'",
        "    ",
        "    "
      ],
      "contextTypes": [
        "  @contextTypes: _.extend {}, Datum.contextTypes,",
        "    # see @proptypes.collection above ",
        "    collection: React.PropTypes.oneOfType([",
        "      React.PropTypes.instanceOf(Backbone.Collection)",
        "      React.PropTypes.string",
        "    ])  "
      ]
    },
    {
      "id": "dd_34",
      "name": "Datum",
      "comment": [
        "",
        "This is base class of all display+input components that render presentation of an attribute from a Backbone.Model or Javascript object.",
        "",
        "There is one required prop, 'attr' - the model attribute name.",
        "",
        "There are two modes of presentation controlled by the `inputMode` prop.  All Datums support `inputMode=\"readonly\"` - the display presentation of the data; and most support `inputMode=\"edit\"` - the input or form presentation of the attribute. The default presentation is `inputMode=\"readonly\"` ",
        "",
        "The backbone model or javascript object to get and set attributes on is specified in the @props.model or @context.model. Note that @props.model always has precendence.",
        "",
        "TODO :  More Examples",
        "",
        "Display attributes from a Backbone model:",
        "",
        "```jsx",
        "render: function(){",
        "",
        "  user = new UserModel()",
        "  user.fetch()",
        "",
        "  // you don't need to wait for the fetch to complete because we are using",
        "  // the **Model** contextual data component below which will rerender when",
        "  // the model triggers 'sync'",
        "",
        "  return (",
        "    <Model model={user}>",
        "      <Datum attr=\"firstName\" label=\"First Name: \"/>",
        "      <Datum attr=\"firstName\" label=\"Last Name: \"/>",
        "      <Datum attr=\"title\"/>",
        "    </Model>",
        "  )",
        "}",
        "```",
        "",
        "Note that for clarity we have provided the **Text** datum which will also ellipsize the display value for you.  You should probably use that instead of using **Datum** directly like the example above.  e.g.",
        "```",
        "  <Text attr=\"firstName\" label=\"FirstName\"/>",
        "```",
        "",
        "#### Validations",
        "",
        "Datums support validations.  All validation methods should return either true or an error message.  All datums should support 'required' validation prop.",
        "",
        "To make a datum required, simply add the required prop to the component.  Ex:",
        "```",
        "  <Text attr=\"name\" required>",
        "```",
        "Validations are run at time of change and the datum element will get an invalid class for optional styling.  An exclamation icon is rendered and has a popup that will show all errors for that input.",
        "",
        "#### ReactBootstrap ",
        "",
        "If ReactBootstrap is available globally, Datum will use that for the ellipsize and validation error popovers.  You can style the popover as you will.  ",
        "",
        "#### Extending ReactDatum.Datum",
        "",
        "Datum, and any of it's descendants, can be extended using either ES6 or Coffeescript inheritance.  The API docs below call out the methods that you will most likely need to override or extend for customization.",
        "",
        "TODO : link to examples - 1 ea. coffeescript and es6",
        "",
        "  ",
        "",
        "",
        "",
        "",
        "",
        "  see ./datum.md"
      ],
      "lineNumber": 13,
      "file": "src/datums/datum.cjsx",
      "signature": "module.exports = class Datum extends React.Component",
      "superClass": " React.Component",
      "methods": [
        {
          "id": "dd_36",
          "name": "componentWillReceiveProps",
          "comment": [
            "      React life cycle methods"
          ],
          "lineNumber": 111,
          "file": "src/datums/datum.cjsx",
          "signature": "  componentWillReceiveProps: (nextProps) ->"
        },
        {
          "id": "dd_37",
          "name": "renderDatumWrapper",
          "comment": [
            "    Rendering methods"
          ],
          "lineNumber": 131,
          "file": "src/datums/datum.cjsx",
          "signature": "  renderDatumWrapper: (contentFn)->"
        },
        {
          "id": "dd_38",
          "name": "renderWrappedDisplayValue",
          "comment": [
            "    In most cases, this is the method you want to override in a custom datum to ",
            "    alter the way the model attribute is displayed when inputMode='readonly'",
            "    ",
            "    This method is only called if the model value is not null.  "
          ],
          "lineNumber": 171,
          "file": "src/datums/datum.cjsx",
          "signature": "  renderWrappedDisplayValue: (value)->"
        },
        {
          "id": "dd_39",
          "name": "renderEllipsizedValue",
          "comment": [
            "    Note that this method is not called by Datum directly.  It is ",
            "    provided here so that any Datum extensions can ellipsize whatever",
            "    part of their rendering neccessary and have a consistent prop and ",
            "    method for doing so."
          ],
          "lineNumber": 187,
          "file": "src/datums/datum.cjsx",
          "signature": "  renderEllipsizedValue: (value, options={}) ->"
        },
        {
          "id": "dd_40",
          "name": "isEditable",
          "comment": [
            "    In most cases this is the method you want to override to alter the presentation of the datum when",
            "    inputMode='edit'"
          ],
          "lineNumber": 259,
          "file": "src/datums/datum.cjsx",
          "signature": "  isEditable: () ->"
        },
        {
          "id": "dd_47",
          "name": "setModelValue",
          "comment": [
            "    do not override this method to return a component element or jsx.  it's bad"
          ],
          "lineNumber": 306,
          "file": "src/datums/datum.cjsx",
          "signature": "  setModelValue: (value, options={}) ->"
        }
      ],
      "propTypes": [
        "  @propTypes:",
        "    # additional css classes (space seperated) to add to wrapper div",
        "    className: React.PropTypes.string",
        "    ",
        "    # can also accept model instance as context var. prop has precendence",
        "    model: React.PropTypes.oneOfType([",
        "      React.PropTypes.instanceOf(Backbone.Model)",
        "      React.PropTypes.object",
        "    ])",
        "    ",
        "    # the backbone attribute on the model to get and set",
        "    attr: React.PropTypes.string.isRequired",
        "    ",
        "    label: React.PropTypes.string",
        "    ",
        "    # set ellipsizeAt to false to display whole value. Only effects 'readonly' display",
        "    # values displayed in 'edit' mode are never truncated.",
        "    ellipsizeAt: React.PropTypes.oneOfType([",
        "      React.PropTypes.number",
        "      React.PropTypes.bool",
        "    ])",
        "    ",
        "    placeholder: React.PropTypes.string",
        "    ",
        "    # TODO : add back support of 'inlineEdit'?",
        "    # 'readonly' = render for display;",
        "    # 'edit' = render for input;",
        "    inputMode: React.PropTypes.oneOf(['readonly', 'edit'])  ",
        "    ",
        "    # set to true to not render a popover on ellipsized values",
        "    noPopover: React.PropTypes.bool",
        "    ",
        "    # set to true to set the model value on change. this defaults to true if inputMode = inlineEdit",
        "    setOnChange: React.PropTypes.bool",
        "    ",
        "    # make this input readonly regardless of context or inputMode prop",
        "    readonly: React.PropTypes.bool",
        "    ",
        "    # make this input required and give it required class and invalid class when invalid",
        "    required: React.PropTypes.bool"
      ],
      "defaultProps": [
        "  @defaultProps:",
        "    # ellipsizeAt is defaulted to prevent really long strings from breaking layouts",
        "    ellipsizeAt: 35"
      ],
      "contextTypes": [
        "  @contextTypes:",
        "    # can also accept model instance as a prop.  prop has precendence",
        "    model: React.PropTypes.oneOfType([",
        "      React.PropTypes.instanceOf(Backbone.Model)",
        "      React.PropTypes.object",
        "    ])",
        "    ",
        "    # note that the readonly prop takes precendence",
        "    inputMode: React.PropTypes.oneOf(['readonly', 'edit', 'inlineEdit'])",
        "    ",
        "    # if there is a form context, we will register with it so that it can",
        "    # interact with us",
        "    form: React.PropTypes.object"
      ]
    },
    {
      "id": "dd_55",
      "name": "Email",
      "comment": [
        "  For rendering and input of email addresses.  Can render mailto: links like ",
        "  `<a href=\"mailto:\">` in display mode",
        "  ",
        "  Validates that email address is a semi valid email based on matching ",
        "  `/^[a-zA-Z0-9_.+-]+@[a-zA-Z0-9-]+\\.[a-zA-Z0-9-.]+$/` "
      ],
      "lineNumber": 13,
      "file": "src/datums/email.cjsx",
      "signature": "module.exports = class Email extends Datum",
      "superClass": " Datum",
      "methods": [],
      "propTypes": [
        "  @propTypes: _.extend {}, Datum.propTypes,",
        "    # If true, display as mailto address link when `inputMode='readonly'`",
        "    displayAsLink: React.PropTypes.bool"
      ]
    },
    {
      "id": "dd_60",
      "name": "LazyPhoto",
      "comment": [
        "  This is a lazy loading image.",
        "",
        "  To prevent a page heavily loaded with images preventing other content from loading, a small",
        "  blank image is downloaded and rendered first and then onLoad the real image src is used and",
        "  rerender.",
        "",
        "  On error a notFoundUrl is set as the image src to prevent broken image display.",
        "",
        "  The model attribute specified in @props.attr should return a fully qualified",
        "  url.  The image is only rendered if it's visible and in view. Otherwise the placeholder",
        "  image is rendered."
      ],
      "lineNumber": 17,
      "file": "src/datums/lazyPhoto.cjsx",
      "signature": "module.exports = class LazyPhoto extends Datum",
      "superClass": " Datum",
      "methods": []
    },
    {
      "id": "dd_65",
      "name": "Link",
      "comment": [
        "",
        "For rendering links like `<a href=\"...\">`.",
        "",
        "Example: ",
        "```jsx",
        "var model = new Backbone.Model({",
        "  url: \"http://www.zulily.com\", ",
        "  name: \"Zulily\"",
        "})",
        "",
        "<ReactDatum.Link attr=\"url\" nameAttr=\"name\"/>",
        "```",
        "",
        "The 'attr' prop should return a url from the model and is the href of the link.",
        "",
        "Optionally, a 'nameAttr' prop can also be specified to display between the <a></a> tags.  If 'nameAttr' prop is not specified, the children of the link are rendered:",
        "```jsx",
        "// we use an alias to minimize the noise in the JSX below",
        "var Rd = ReactDatum;",
        "",
        "// ... and later ...",
        "<Rd.Link attr=\"url\">Go to <Rd.Text attr=\"name\"/> and discover somthing today!</Rd.Link>",
        "```",
        "",
        "In this example the link would enclose the rendered contents: \"Go to Zulily and ...\"",
        "",
        "If no nameAttr prop and no children, ReactDatum.Link will render the value of the attr specified (what should be the url of the href):",
        "```jsx",
        "<Rd.Link attr=\"url\"/>",
        "```",
        "renders `<a href=\"http://www.zulily.com\">http://www.zulily.com</a>`  ",
        "",
        "  see ./link.md"
      ],
      "lineNumber": 10,
      "file": "src/datums/link.cjsx",
      "signature": "module.exports = class Link extends Datum",
      "superClass": " Datum",
      "methods": [],
      "propTypes": [
        "  @propTypes: _.extend {}, Datum.propTypes,",
        "    # attribute on model to display as the <a> content. if null ReactDatum.Link will ",
        "    # render the children enclosed in an <a></a>",
        "    nameAttr: React.PropTypes.string",
        "    # passed to <a> as the target",
        "    target: React.PropTypes.string",
        "    ",
        "    "
      ],
      "defaultProps": [
        "  @defaultProps: _.extend {}, Datum.defaultProps,",
        "    target: '_blank'",
        "    "
      ]
    },
    {
      "id": "dd_69",
      "name": "Number",
      "comment": [
        "  For real numbers.",
        "",
        "  Only allows `/^\\-?[0-9]*\\.?[0-9]*$/` on input"
      ],
      "lineNumber": 15,
      "file": "src/datums/number.cjsx",
      "signature": "module.exports = class Number extends Datum",
      "superClass": " Datum",
      "methods": [],
      "propTypes": [
        "  @propTypes: _.extend {}, Datum.propTypes,",
        "    # 'abbreviate' - Add M and K to numbers greater than 1 million and 1 thousand respectively",
        "    # 'money' - display dollar sign and two decimal places zero filled",
        "    # 'comma' - add comma separators at thousands",
        "    format: React.PropTypes.oneOf(['abbreviate','money','comma'])",
        "    ",
        "    #validate value is at least this value on change",
        "    minValue: React.PropTypes.number",
        "    ",
        "    #validate value is at most this value on change",
        "    maxValue: React.PropTypes.number"
      ]
    },
    {
      "id": "dd_76",
      "name": "Text",
      "comment": [
        "",
        "This react-datum component is for the presentation of text. See also ReactDatum.Datum. The Datum base class does much of the work by default of handling text. ",
        "",
        "Text Component provides the following additional features:",
        "- Display values can be ellipsized via ellipsizeAt prop.  ",
        "- Values are ellipsized by default to prevent unexpected long strings from breaking your layout",
        "- Can display HTML via the displayAsHtml prop",
        "",
        "**A note about using displayAsHtml**",
        "",
        "It's not advised unless you are sure as to the sanity of the data value you are displaying.  The danger is in cross site scripting attacks.  Read more [here](https://facebook.github.io/react/tips/dangerously-set-inner-html.html).",
        "",
        "Also note that using `displayAsHtml` and anything other than `ellipsizeAt={false}` is not advised as you may end up ellipsizing in the middle of an element and leave unclosed tags or, worse, break your entire layout. ",
        "",
        "For example, given this model:",
        "```jsx",
        "  model = new Backbone.model({name=\"Foo Bar\", description=\"Lorem ipsum <b>dolor sit amet</b>, consectetur adipiscing elit. Duis rhoncus lacinia lectus a volutpat. this could go on forever\"});",
        "```",
        "this will render the description truncated with ellipsis to not exceed 35 total characters:",
        "```",
        "  <ReactDatum.Text attr=\"description\">",
        "```",
        "this will render the description at it's full length:",
        "```",
        "<ReactDatum.Text attr=\"description\" ellipsizeAt={false}>",
        "```",
        "this will render the description with \"dolor sit amet\" in bold:",
        "```",
        "<ReactDatum.Text attr=\"description\" displayAsHtml ellipsizeAt={false}>",
        "```",
        "",
        "",
        "",
        "  see ./text.md"
      ],
      "lineNumber": 10,
      "file": "src/datums/text.cjsx",
      "signature": "module.exports = class Text extends Datum",
      "superClass": " Datum",
      "methods": [],
      "propTypes": [
        "  @propTypes: _.extend {}, Datum.propTypes,",
        "    # set to true if rendering known, safe, html.  see https://facebook.github.io/react/tips/dangerously-set-inner-html.html. ",
        "    displayAsHtml: React.PropTypes.bool",
        "    ",
        "    # set ellipsizeAt to false to display whole value. Only effects 'readonly' display",
        "    # values displayed in 'edit' mode are never truncated.",
        "    ellipsizeAt: React.PropTypes.oneOfType([",
        "      React.PropTypes.number",
        "      React.PropTypes.bool",
        "    ])"
      ],
      "defaultProps": [
        "  @defaultProps: _.extend {}, Datum.defaultProps,",
        "    # ellipsizeAt is defaulted to prevent really long strings from breaking layouts",
        "    ellipsizeAt: 35",
        "    "
      ]
    },
    {
      "id": "dd_81",
      "name": "WholeNumber",
      "comment": [
        "  For whole numbers (no decimal input allowed)."
      ],
      "lineNumber": 7,
      "file": "src/datums/wholeNumber.cjsx",
      "signature": "module.exports = class WholeNumber extends Number",
      "superClass": " Number",
      "methods": []
    },
    {
      "id": "dd_112",
      "name": "Form",
      "comment": [
        "",
        "This contextual component provides a form context to all of the datums within.  ",
        "",
        "Adding the **ReactDatum.Form** component, you can easily convert a group of datums into an editable form with save and cancel buttons.  By default, when children of a Form component all datums will render in their input presentation `inputMode='edit'`.",
        "",
        "<img alt=\"Screenshot from doc/examples/form/form.html\" src=\"https://gitlab.corp.zulily.com/bwilkerson/react-datum/raw/master/img/react-datum_form-example.png\" align=\"right\"/>",
        "",
        "```javascript",
        "var kittenCard = React.createClass({",
        "  displayName:\"KittenCard\",",
        "  render: function(){",
        "    return (",
        "      <div className='kitten-card'>",
        "        <ReactDatum.Model model={kittenModel}>",
        "          <h3>Adopt <ReactDatum.Text attr=\"name\"/> Today!</h3>",
        "          <ReactDatum.Form>",
        "            <div><ReactDatum.LazyPhoto attr=\"imgUrl\"/></div>",
        "            <div><ReactDatum.Text attr=\"name\" label=\"Name\" setOnChange/> (<ReactDatum.Text attr=\"title\"/>)</div>",
        "            <label>Say something about <ReactDatum.Text attr=\"name\" readonly/>: </label>",
        "            <div><ReactDatum.Text attr=\"description\" className=\"wide-input\"/></div>",
        "            <div><ReactDatum.Email attr=\"sponsorEmail\" label=\"Adoption Sponsor\" displayLink/></div>",
        "            <label>Leave a Comment!</label>",
        "            <div><ReactDatum.Text attr=\"comment\" className=\"wide-input\"/></div>",
        "          </ReactDatum.Form>",
        "        </ReactDatum.Model>",
        "      </div>",
        "    )",
        "  }",
        "})",
        "```",
        "",
        "The **ReactDatum.Form** component adds save and cancel buttons.  When the user presses save, model.save() is called. All of the attributes were set() when the user entered new values If cancel is clicked, the model and subsequently, the form are reset back to the state of the last model.save().",
        "",
        "By wrapping the datums in the **ReactDatum.Form** tag, they implicitedly recieve `inputMode='edit'` props that make them all render as inputs.  Almost all.  Some Datums, like **ReactDatum.LazyPhoto**, only have a display presentation, no update.  If given an `inputMode='edit'` they will ignore, and continue showing their display (`inputMode='readonly'``) representation.  ",
        "",
        "",
        "",
        "",
        ""
      ],
      "lineNumber": 8,
      "file": "src/form.cjsx",
      "signature": "module.exports = class Form extends React.Component",
      "superClass": " React.Component",
      "methods": [
        {
          "id": "dd_114",
          "name": "renderButtonContainer",
          "comment": [
            "    Gives the first editable datum focus"
          ],
          "lineNumber": 131,
          "file": "src/form.cjsx",
          "signature": "  renderButtonContainer: (options={}) ->",
          "defaultOptions": [
            "    options = _.defaults options,",
            "      addClass: null"
          ]
        },
        {
          "id": "dd_117",
          "name": "save",
          "comment": [
            "    Save the changes from datums on the form to the Backbone model. ",
            "    ",
            "    Calls model.save after first attempting to validate() the model.  Handles ",
            "    inconsistencies in model.validate() between versions 0.9.2 - 1.2.2 of Backbone.",
            "    ",
            "    The user clicking on the save button belonging to the Form will call this Method",
            "    ",
            "    The options argument is passed on to Backbone model.save()"
          ],
          "lineNumber": 181,
          "file": "src/form.cjsx",
          "signature": "  save: (options={})->",
          "defaultOptions": [
            "    options = _.defaults options,",
            "      validateDatums: true      # TODO : these should also be @props",
            "      validateModel: true",
            "      ",
            "    @setState errorMessage: null, successMessage: null",
            "    model = @getModel()",
            "    ",
            "    if options.validateDatums and not @validateDatums(options)",
            "      @onSaveError model, \"Correct errors and try again.\"",
            "      return"
          ]
        },
        {
          "id": "dd_118",
          "name": "validateDatums",
          "comment": [
            "    Validate the datums on the form. ",
            "    ",
            "    returns false if any are currently invalid"
          ],
          "lineNumber": 206,
          "file": "src/form.cjsx",
          "signature": "  validateDatums: (options={}) ->"
        },
        {
          "id": "dd_119",
          "name": "validateModel",
          "comment": [
            "    Calls Backbone model.validate and handles inconsistencies in model.validate() ",
            "    between versions 0.9.2 - 1.2.2 of Backbone."
          ],
          "lineNumber": 218,
          "file": "src/form.cjsx",
          "signature": "  validateModel: (options={}) ->"
        },
        {
          "id": "dd_121",
          "name": "saveModel",
          "comment": [
            "    calls Backbone model.save and calls success and error handlers. ",
            "    ",
            "    You should probably call Form.save() above instead.  It will also validate the model ",
            "    and datums.  "
          ],
          "lineNumber": 254,
          "file": "src/form.cjsx",
          "signature": "  saveModel: (options={}) ->"
        },
        {
          "id": "dd_126",
          "name": "addDatum",
          "comment": [
            "    This method is called by the datum children when they mount"
          ],
          "lineNumber": 304,
          "file": "src/form.cjsx",
          "signature": "  addDatum: (datumComponent) ->"
        },
        {
          "id": "dd_127",
          "name": "removeDatum",
          "comment": [
            "    This method is called by the datum children when they unmount"
          ],
          "lineNumber": 311,
          "file": "src/form.cjsx",
          "signature": "  removeDatum: (datumComponent) ->"
        }
      ],
      "propTypes": [
        "  @propTypes:",
        "    # can also accept model instance as context var. prop has precendence ",
        "    model: @modelOrObject()",
        "    ",
        "    #  no formMode like zform, but we have to support programatic readonly",
        "    #  see also ClickToEditForm component.   readonly should always take precendence",
        "    readonly: React.PropTypes.bool",
        "    ",
        "    # you can style a buttonPossiton: 'top' to float left or right in css.",
        "    buttonPosition: React.PropTypes.oneOf(['top', 'bottom', 'none'])",
        "    ",
        "    # specify className of form element",
        "    className: React.PropTypes.string",
        "    ",
        "    # on save success this method, if specified, will be called with the standard",
        "    # Backbone success callback arguments (model, response, options)",
        "    # If you don't specify a saveSuccessCallback, a small success message will be rerendered",
        "    # below the form button after successfully saving the form",
        "    saveSuccessCallback: React.PropTypes.func",
        "    ",
        "    # on save error this method, if specified, will be called with the standard",
        "    # Backbone error callback arguments (model, response, options)",
        "    # If you don't specify a saveErrorCallback, an error message will be rerendered",
        "    # below the form button after failing to save the form on user request",
        "    saveErrorCallback: React.PropTypes.func"
      ],
      "defaultProps": [
        "  @defaultProps:",
        "    readonly: false",
        "    buttonPosition: 'bottom'",
        "    className: 'zform'"
      ],
      "contextTypes": [
        "  @contextTypes:",
        "    ### can also accept model instance as a prop.  prop has precendence ###",
        "    model: @modelOrObject()",
        "    inputMode: Datum.contextTypes.inputMode"
      ]
    },
    {
      "id": "dd_144",
      "name": "SelectableCollection",
      "comment": [
        "  This collection instance mixin provides the ability to mark models as selected and active.",
        "",
        "  There is one active model at a time and many selected. The collection will trigger an \"activeModelChanged\"",
        "  event when the active model is set via setActiveModel() method.  Current version of this component does not",
        "  support having active model that is not selected.  Calling setActiveModel on an unselected model, selects it.",
        "",
        "  example:",
        "  ```javascript",
        "    kittensCollection = new Backbone.Collection()",
        "    SelectableCollection.mixInto(kittensCollection)",
        "    kittensCollection.onSelectionsChanged(function(){",
        "      alert(\"you selected \" + kittensCollection.getSelectedModels().length + \" kittens\")",
        "    })",
        "    ...",
        "",
        "    kittensCollection.selectModelByIndex(0)",
        "    ...",
        "  ```",
        "  When a collection is reset([]), or a selected model is removed from the collection it is no longer returned",
        "  by any of the getSelected... methods.  Only models that exist in the collection can be selected.",
        "",
        "  When a model is selected, model.selected=true.",
        "",
        "  Events triggered on collection:",
        "",
        "    selectionsChanged       - triggered whenever selections change",
        "    activeModelChanged      - function(activeModel){} triggered on active change",
        ""
      ],
      "lineNumber": 32,
      "file": "src/mixins/selectableCollection.coffee",
      "signature": "module.exports = class SelectableCollection",
      "methods": [
        {
          "id": "dd_145",
          "name": "@mixInto",
          "comment": [
            "    This method is used to mix SelectableCollection features into a Backbone Collection"
          ],
          "lineNumber": 37,
          "file": "src/mixins/selectableCollection.coffee",
          "signature": "  @mixInto: (collection) ->"
        }
      ]
    },
    {
      "id": "dd_158",
      "name": "Model",
      "comment": [
        "",
        "This react-datum component provides datums with a Backbone model and rerenders descendants on model changes.  ",
        "",
        "Any datum component like [ReactDatum.Text](TODO: add a link do doc page), will accept the Backbone model provided by this class as a context variable.  Outside of a **ReactDatum.Model*** component, datums will not respond to model changes.",
        "",
        " ",
        ""
      ],
      "lineNumber": 9,
      "file": "src/model.cjsx",
      "signature": "module.exports = class Model extends ContextualData",
      "superClass": " ContextualData",
      "methods": [],
      "propTypes": [
        "  @propTypes: _.extend {}, ContextualData.propTypes,",
        "    model: React.PropTypes.oneOfType([",
        "      React.PropTypes.instanceOf(Backbone.Model)",
        "      React.PropTypes.func",
        "    ]).isRequired"
      ]
    },
    {
      "id": "dd_161",
      "name": "SelectedModel",
      "comment": [
        "",
        "This react-datum contextual component provides datums with a Backbone model that is selected model in the collection.  It also  rerenders descendants on collection selection changes.  ",
        "",
        "Any descendant datum that accept a `model` context or prop, can be wrapped in a **ReactDatum.SelectedModel** component and will respond to changes in the model and on changes to the selected model in the collection.",
        "",
        "Simple example:",
        "```jsx",
        "  var Rd = ReactDatum;",
        "  ",
        "  <Rd.SelectedModel collection={myBackboneCollection} placeholder=\"Wait for it...\">",
        "    <Rd.Text label=\"You selected: \" attr=\"name\"/>",
        "  </Rd.SelectedModel>",
        "  ",
        "  var index = 0;",
        "  setInterval(function(){",
        "    myBackboneCollection.selectModelByIndex(index);",
        "    index += 1;",
        "  }, 5000)",
        "",
        "```",
        "The (example above)[TODO: add link to github.io example] when run, will initially display a placeholder, \"Wait for it...\" and then every five seconds select the next model in the collection. ",
        "",
        "The placeholder prop will render in place of any children when there is no selected model in the collection and can be any React renderable.",
        "",
        "#### SelectableCollection mixin",
        "",
        "Use of the SelectedModel component will automatically add the ReactDatum.SelectableCollection to the collection instance unless collection.hasSelectableCollectionMixin is true.  **setModelByIndex(index)** in the example above is one of the methods added by that mixin. See (ReactDatum.SelectableCollection)[TODO: add doc link]",
        "",
        "",
        ""
      ],
      "lineNumber": 6,
      "file": "src/selectedModel.cjsx",
      "signature": "module.exports = class SelectedModel extends ContextualData",
      "superClass": " ContextualData",
      "methods": [],
      "contextTypes": [
        "  @contextTypes:",
        "    collection: React.PropTypes.instanceOf(Backbone.Collection)"
      ]
    },
    {
      "id": "dd_164",
      "name": "Tilegrid",
      "comment": [
        "",
        "The Tilegrid component renders a tile containing it's children for each model in the collection.  ",
        "",
        "- supports single select ",
        "- supports multi/drag select",
        "- lazily renders tiles",
        "- lazily derenders tiles outside the view port",
        "- supports variable height and width tiles within the same grid",
        "- supports nested Tilegrid components",
        " ",
        "simple example:",
        "```jsx",
        "// KITTEN_DATA is a static array of data from petfinder api",
        "//    that gets loaded via script tag for the examples",
        "kittenCollection = new Backbone.Collection(KITTEN_DATA)",
        "",
        "var TilegridDisplay = React.createClass({",
        "  displayName:\"TilegridDisplay\",",
        "  render: function(){",
        "    return (",
        "      <Rd.Collection collection={kittenCollection}>",
        "        <Rd.Tilegrid>",
        "          <Rd.LazyPhoto attr=\"imageUrl\"/>",
        "          <h4><Rd.Text attr=\"name\"/></h4>",
        "          <Rd.Email attr=\"breed\"/>",
        "        </Rd.Tilegrid>",
        "      </Rd.Collection>",
        "    );",
        "    ",
        "```",
        ""
      ],
      "lineNumber": 9,
      "file": "src/tilegrid.cjsx",
      "signature": "module.exports = class Tilegrid extends React.Component",
      "superClass": " React.Component",
      "methods": [],
      "propTypes": [
        "  @propTypes:",
        "    # can also accept collection through the @context.collection (see, widgets.react.Collection)",
        "    # can accept any of the acceptable Tilegrid constructor arg 'data'",
        "    collection: React.PropTypes.any",
        "    # tileTemplate can be passed by prop or by virtue of a <Tile> child element.  This can also be a",
        "    # string template to render oldschool style.  Note that tileTemplate prop has precendence",
        "    tileTemplate: React.PropTypes.node",
        "    # if you have an custom tilegrid to use like that in bundles, this may work, but only if your",
        "    # constructor arguments are in sync with those of base Tilegrid class",
        "    # (note: func is a class)",
        "    tilegridClass: React.PropTypes.func",
        "    # see widgets.Tilegrid constructor",
        "    tilegridOptions: React.PropTypes.object"
      ],
      "defaultProps": [
        "  @defaultProps:",
        "    tilegridClass: TilegridReact",
        "    tileTemplate: null",
        "    tilegridOptions: {}",
        "    tilegridSelectionClass: SingleSelect"
      ],
      "contextTypes": [
        "  @contextTypes:",
        "    # can accept any of the acceptable Tilegrid constructor arg 'data'",
        "    collection: React.PropTypes.any"
      ]
    },
    {
      "id": "dd_167",
      "name": "MultiSelect",
      "comment": [
        "  This class implements the methods necessary to connect a Tilegrid widget to a Collection with the",
        "  SelectableCollection mixin and provides a multi select strategy for Tilegrid.",
        "",
        "  Clicking a tile selects the corresponding model in the collection throuh selectableCollection#selectModel...",
        "  interfaces.  Clicking and draging selects all models between the drag start and drag end tile.",
        "",
        "  - Holding down the command toggles rows to the selected.",
        "  - Holding down the shift key extends the selection from the active cell to the clicked cell",
        "  - Dragging always extends away and back to the first row selected in the drag operation",
        "",
        "  Usage:  simply construct an instance of this class any time after constructing a Tilegrid and pass the tilegrid",
        "  instance to MultiSelect constructor:",
        "",
        "    new MultiSelect(@tilegrid)",
        ""
      ],
      "lineNumber": 23,
      "file": "src/tilegrid/multiSelect.coffee",
      "signature": "module.exports = class MultiSelect extends SingleSelect",
      "superClass": " SingleSelect",
      "methods": []
    },
    {
      "id": "dd_173",
      "name": "SingleSelect",
      "comment": [
        "  This class implements the methods necessary to connect selections in a Tilegrid widget to a Collection with the",
        "  SelectableCollection mixin.",
        "",
        "  This is the implementation of a single select strategy for Tilegrid.  Clicking a tile selects the",
        "  corresponding model in the collection throuh selectableCollection#selectModel interfaces.",
        "",
        "  See widgets.tilegrid.MultiSelect for a multiselect strategy that supports selecting multiple models",
        "  and clicking and dragging to select multiple models.",
        "",
        "  Usage:  simply construct an instance of this class any time after constructing a Tilegrid and pass the tilegrid",
        "  instance to SingleSelect constructor:",
        "",
        "    new SingleSelect(@tilegrid)",
        "",
        "  **Note for React users**: this class is initialized by the selection='single' prop on <Rb.Tilegrid>",
        "",
        "  Note that constructing this class on a Tilegrid with a collection will cause the selectableCollection",
        "  mixin to be added to the collection.  See mixins/SelectableCollection",
        ""
      ],
      "lineNumber": 29,
      "file": "src/tilegrid/singleSelect.coffee",
      "signature": "module.exports = class SingleSelect",
      "methods": []
    },
    {
      "id": "dd_214",
      "name": "TilegridReact",
      "comment": [
        "  this extension of the tilegrid allows the use of ReactComponents as the tile template.",
        "",
        "  See src/tilegrid for tile grid that can be used as a React component",
        "  from JSX",
        "",
        "  TODO : fold this functionality into ./tilegrid.coffee"
      ],
      "lineNumber": 17,
      "file": "src/tilegrid/tilegridReact.coffee",
      "signature": "module.exports = class TilegridReact extends Tilegrid",
      "superClass": " Tilegrid",
      "methods": []
    }
  ],
  "methods": [
    {
      "id": "dd_83",
      "name": "highlight",
      "comment": [],
      "lineNumber": 10,
      "file": "src/docs/api/apiDocs.cjsx",
      "signature": "  highlight: (code) ->"
    },
    {
      "id": "dd_84",
      "name": "renderLinks",
      "comment": [],
      "lineNumber": 75,
      "file": "src/docs/api/apiDocs.cjsx",
      "signature": "  renderLinks: (code)->"
    },
    {
      "id": "dd_133",
      "name": "hasScroll",
      "comment": [
        "  Thanks stackoverflow!   This allows the tilegrid to determine the scroll parent of the tiles",
        "  http://codereview.stackexchange.com/questions/13338/hasscroll-function-checking-if-a-scrollbar-is-visible-in-an-element",
        "",
        "  You can then use this in any jQuery selector, such as:",
        "  ```",
        "    $('div:hasScroll') //all divs that have scrollbars",
        "    $('div').filter(':hasScroll') //same but better",
        "    $(this).closest(':hasScroll(y)') //find the parent with the vert scrollbar",
        "    $list.is(':hasScroll(x)') //are there any horizontal scrollbars in the list?",
        "  ```"
      ],
      "lineNumber": 14,
      "file": "src/lib/jqueryHelpers.coffee",
      "signature": "hasScroll = (el, index, match) ->"
    }
  ],
  "name": "react-datum"
};
